######## Function bin_matrix ########
# A function that takes all objects as input to create three
# m x n matrices X_formation, X_thickness, X_depth
# m = number of wells in data set
# n = number of formations existing in NPD database, which is 212
# Each well (object) has a certain set of fm layers that make up
# the total geological profile. These layers have unique IDs defined
# by NPD. X_formation, X_thickness, and X_depth stores information
# about if the fm exists in object, thickness of fm, and depth (TVD)
# of top of fm respectively.
# Example; If well i has the layer Nordland with unique ID 113.
# X_formation gets a binary 1 in the ith row and 113th column.
# 0 is given if the fm does not exist. For X_thickness and X_depth,
# instead of binary 0/1, we store the information directly into the
# ith row and 113th column. The purpose, is to have a place where
# information is easily stored and traceable since we always have
# that the row index refer to a well, while the column index refers
# to a given fm layer.

# Parameters
# ----------------
# Objects : list
#   The number of objects (wells) that is available in the current
#   database.

import numpy as np

def bin_matrix(Objects):
    dim = (len(Objects), 212)
    X_formation = np.zeros(dim)
    X_thickness = np.zeros(dim)
    X_depth = np.zeros(dim)
    temp = []
    for i in range(len(Objects)):
        for j in range(len(Objects[i].top)):
            X_formation[i][Objects[i].fm[j]] = 1
            layer = Objects[i].bottom[j] - Objects[i].top[j]
            X_thickness[i][Objects[i].fm[j]] = layer
            temp.append(layer)
            X_depth[i][Objects[i].fm[j]] = Objects[i].top[j]
        Objects[i].thickness = temp
        temp = []
    return X_formation, X_thickness, X_depth, Objects

