######## The Expert System ########
# Shall select relevant wells for a new well that is to be drilled.
#
# In total, the expert system is expressed by the following scripts:
#   - main.py
#   - bin_matrix.py
#   - formation.py
#   - mod_kNN.py
#   - prompt_weights.py
#   - raw_input.py
#   - relevancy.py
#   - scaling.py
#   - standard_dev.py
#   - target_cluster.py
###################################

## General import
import os
import pandas as pd
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

## Import from Scikit-learn
from sklearn.cluster import k_means as kMeans
from sklearn import manifold
from sklearn.metrics.pairwise import pairwise_distances

## Define class Well to make objects
class Well:
    def __init__(self,name=None,index=None,top=None,bottom=None,fm=None,
                 fm_names=None,thickness=None,thickness_sum=None,
                 fm_rel=None,thickness_rel=None,depth_rel=None):
        self.name = name                    # Name of well
        self.index = index                  # Index of well
        self.top = top                      # Top of fm, TVD
        self.bottom = bottom                # Bottom of fm, TVD
        self.fm = fm                        # Unique number of fm
        self.depth = self.bottom[-1]        # Total depth, TVD
        self.fm_names = fm_names            # Name of fm
        self.thickness = thickness          # Thickness of fm
        #self.fm_rel = fm_rel               # Integer,
        #self.thickness_rel = thickness_rel # Integer,
        #self.depth_rel = depth_rel         # Integer,

    def __repr__(self):
        return '{}'.format(self.name)

## Empty lists
a = []                                      # Top of fms, TVD
b = []                                      # Bottom of fms, TVD
c = []                                      # Name of fms
d = []                                      # Unique number of fms
a_temp = []
b_temp = []
c_temp = []
d_temp = []
ref = []                                    # Name of wells
input_well = False

######## Part 1: Pre-processing of input data
## Retrieve input well
os.chdir('C:\\Users\MinhHoan\OneDrive – NTNU\Prosjekt og master\
Masteroppgaven\Python\Input')

if os.listdir() == []:
    pass
else:
    input_well = True
    input_file = pd.read_excel('Input.xls', usecols=[0,1,2,5,3],
                               names=[0,1,2,3,4],skip_footer=8363)

for i in range(len(input_file)):
    if i+1 not in range(len(input_file)):
        a_temp.append(input_file[1][i])
        b_temp.append(input_file[2][i])
        c_temp.append(input_file[3][i])
        d_temp.append(input_file[4][i])
        a.append(a_temp)
        b.append(b_temp)
        c.append(c_temp)
        d.append(d_temp)
        a_temp = []
        b_temp = []
        c_temp = []
        d_temp = []
        ref.append(input_file[0][i])
    elif input_file[1][i] <= input_file[1][i + 1]:
        a_temp.append(input_file[1][i])
        b_temp.append(input_file[2][i])
        c_temp.append(input_file[3][i])
        d_temp.append(input_file[4][i])

target_depth = b[0][-1]                        # Total depth, TVD

## Retrieve the database that is available from NPD
os.chdir('C:\\Users\MinhHoan\OneDrive – NTNU\Prosjekt og master\
Masteroppgaven')

Data = pd.read_excel('Datasett.xls', usecols=[0,1,2,5,3],
                     names=[0,1,2,3,4])
for i in range(len(Data)):
    if i+1 not in range(len(Data)):
        a.append(a_temp)
        b.append(b_temp)
        c.append(c_temp)
        d.append(d_temp)
        ref.append(Data[0][i])
    elif Data[1][i] <= Data[1][i+1]:
        a_temp.append(Data[1][i])
        b_temp.append(Data[2][i])
        c_temp.append(Data[3][i])
        d_temp.append(Data[4][i])
    else:
        a_temp.append(Data[1][i])
        b_temp.append(Data[2][i])
        c_temp.append(Data[3][i])
        d_temp.append(Data[4][i])
        a.append(a_temp)
        b.append(b_temp)
        c.append(c_temp)
        d.append(d_temp)
        a_temp = []
        b_temp = []
        c_temp = []
        d_temp = []
        ref.append(Data[0][i])

## Removing layers that are categorized as No formal name or
# Undifferentiated
i = 0
while i < len(d[0]):
    if d[0][i] == 111 or i == 182:
        a[0].pop(i)
        b[0].pop(i)
        c[0].pop(i)
        d[0].pop(i)
        i += 1
    else:
        i += 1

## Generate objects based on the defined class, Well
Objects = []
for i in range(len(a)):
    Objects.append(Well(top=a[i], bottom=b[i], name=ref[i],fm=d[i],
                        index=i,fm_names=c[i]))

######## PART 2: MACHINE LEARNING METHODS - KMEANS

## Make input data ready for k-means and multidimensional scaling
from bin_matrix import bin_matrix
X_formation,X_thickness, X_depth, Objects = bin_matrix(Objects)

## Call function cluster to prepare input data for k-means
from raw_input import cluster
num_clusters = 120
raw_input = cluster(X_thickness,X_depth,d)

## k-means
kmeans_raw = kMeans(raw_input,n_clusters=num_clusters,return_n_iter=True)

## Call function target_cluster to find cluster of which the input well
# lies in, the target cluster
from target_cluster import target_cluster
clusters, clust_target = target_cluster(kmeans_raw,num_clusters)

## Lets examine the total TVD of all wells in the target cluster.
# We remove wells that are much deeper than the input well
if input_well == True:
    depths = []
    pool = []
    for i in clusters[clust_target]:
        depths.append(Objects[i].depth)
        pool.append(Objects[i])
spekter = range(depths[0] - 500, depths[0] + 500)
j = 0
while j < len(depths):
    if depths[j] not in spekter:
        depths.pop(j)
        throw = pool.pop(j)
        j += 1
    else:
        j += 1

######## PART 3: MACHINE LEARNING METHODS - NEAREST NEIGHBOR
## Call function SD to get standard deviatons of fm layers
from standard_dev import SD
SD_thickness, SD_depth, mean_thickness, mean_depth = SD(X_thickness,
                                                        X_depth,pool)

## Call function scaling to get zero mean and unit variance over the
# distribution of each fm layer
from scaling import scaling
input_matrix = scaling(pool, X_thickness, X_depth, SD_thickness,
                       SD_depth, mean_thickness, mean_depth)

## Call function prompt_weights to
# enable engineer's ability to weight fm layers
from prompt_weights import prompt_weights
input_matrix = prompt_weights(pool,input_matrix)

## Find the nearest neighbors of input well and rank them
# according to their distances
from mod_kNN import kNN
pool = kNN(input_matrix,pool)
def sort_func(Well):
    return Well.nn
pool_final = sorted(pool, key=sort_func)

## Use MDS to visualize data
dmatrix = pairwise_distances(raw_input)
mds = manifold.smacof(dissimilarities=dmatrix,n_components=3,
                      metric=True,max_iter=3000,random_state=1,
                      eps=1e-9,normalize=True)
# Plot - 3D space
fig = plt.figure()
ax = Axes3D(fig)
ax.scatter(mds[0][:, 0], mds[0][:, 1], mds[0][:, 2])
ax.set_xlabel('DIM1')
ax.set_ylabel('DIM2')
ax.set_zlabel('DIM3')
plt.show()
# Plot - 2D space
plt.scatter(mds[0][:, 0], mds[0][:, 1])
plt.xlabel('DIM1')
plt.ylabel('DIM2')
plt.show()

## Print outpust of expert system
print('Target well: ', pool_final[0].name)
print('Well name', 'Nearest Neighbor Value')
for i in range(1,len(pool_final)):
    print(pool_final[i].name, pool_final[i].nn)

## Find nearest cluster by locating centroids
centroid_distances = pairwise_distances(kmeans_raw[0],kmeans_raw[0])
j = 10000000
count = 0
for i in range(len(centroid_distances[clust_target])):
    if centroid_distances[clust_target,i] < j and \
            centroid_distances[clust_target,i] > 0:
        j = centroid_distances[clust_target,i]
        count = i
    else:
        continue

print('Nearest centroid has the following wells:')
print('Well name')
for x in clusters[count]:
    print(Objects[x].name)
print('Stress-1:', mds[1])


