######## Function prompt_weights ########
# A function that enables an engineer to weight certain fm layers over
# the others. Weighting is triggered by binary 0/1 for NO/YES.
# We have two weighting arrays: W and w.
# W weights how we should prioritize thickness in fm layers
# in relation to the offset in top of fm.
# w weights how we should prioritize the different fm layers
# in relation to each other. For example, if there exists more
# complicated fm layers (drilling wise) we should weight it
# higher than others to penalize any deviation.

# Parameters
# ----------------
# input_matrix : ndarray
#   A i x 2n matrix. Where i is the number of wells that lies within
#   the target cluster. n is the number of fm layers that the input
#   well consists of.
#
# pool : list
#   The pool of wells. It is the wells lying within the target cluster
#   as objects.

import numpy as np

def prompt_weights(pool,input_matrix):
    print('Your target well has the following layers:')
    for i in pool[0].fm_names:
        print(i)
    print('There are a total of', len(pool[0].fm_names), 'layers.')
    print('Weighting: Layers that are weighted with high values will '
          'be prioritized. Layers are weighted in relation to each '
          'other.')
    prompt = int(input('If you wish to weight layers please select: 1.'
                       'If not, choose default by selecting: 0. '
                       'Type your selection: '))
    weights_thickness = []
    weights_depth = []
    if prompt == 1:
        # Assigning weights to layers by prompt
        weighting = list(input('Weight layers by same ordering as '
                               'given above. No spacing.'))
        # Check that weighting dimensions are correct
        while True:
            if len(pool[0].fm_names) == len(weighting):
                W = np.ones(2,dtype=int)
                w = np.asarray([int(i) for i in weighting])
                for i in range(len(w)):
                    weights_thickness.append(w[i]*W[0])
                    weights_depth.append(w[i]*W[1])
                weights_thickness.extend(weights_depth)
                weights = weights_thickness
                for i in range(len(pool)):
                    for j in range(len(input_matrix[0])):
                        input_matrix[i,j] = weights[j] * input_matrix[i,j]
                break
            else:
                weighting = list(input('Number of weights do not match. '
                                       'No spacing. Try again: '))
    else:
        W = np.array([1,1])
        w = np.ones(len(input_matrix[0]),dtype=int)
        for i in range(int(len(w)/2)):
            weights_thickness.append(w[i] * W[0])
            weights_depth.append(w[i] * W[1])
        weights_thickness.extend(weights_depth)
        weights = weights_thickness
        for i in range(len(pool)):
            for j in range(len(input_matrix[0])):
                input_matrix[i, j] = weights[j] * input_matrix[i, j]
    return input_matrix
