######## Function cluster ########
# Shall prepare the necessary input that is needed to
# perform k-means on the data that was originally taken
# as input. X_thickness, X_depth, and d is taken as input.
# The function uses d to locate the fm layers that exists
# in the input well. It then extracts all information from
# every well in the database in these exact layers, ignoring
# any other fm layers that a well may have.
# The output, raw_input is a m x 2n matrix that is ready for
# k-means. m is the number of wells in the database. n is the
# number of layers our input well has. We have 2n since we want
# to look at both fm thickness and depth of top of fm.

# Parameters
# ----------------
# thickness : ndarray
#   A m x i matrix. m is the number of wells in the database.
#   i the number of fm layers that exists in the database.
#   It carries the information of thickness of each fm layer for
#   every well.
#
# depth : ndarray
#   A m x i matrix. m is the number of wells in the database.
#   i the number of fm layers that exists in the database.
#   It carries the information of TVD top of fm of each layer
#   for every well.
#
# fm : list
#   The lists holds the geological profile of each well in the
#   database, that is, what fm layers a well consists of.

def cluster(thickness,depth,fm):
    raw_input = []
    for i in range(len(thickness)):
        temp = []
        temp2 = []
        for j in fm[0]:
            temp.append(thickness[i,j])
            temp2.append(depth[i,j])
        temp.extend(temp2)
        raw_input.append(temp)
    return raw_input
