######## Function scaling ########
# Shall standardize the well data of the wells within
# the target cluster. By having zero mean and unit variance, the
# the subtle valued thickness in fm are enhances while at the same
# time, the dominating values of depth of top fm are dampened.
# This makes it easier to assess the true deviation when we look for
# relevant wells. It takes the standard deviation of each layer, one
# for thickness and one for depth of top fm as input.
# It produces a input_matrix that has same length as pool.
# The input_matrix is used in kNN to rank the wells lying in the
# target cluster after least to most deviation.

# Parameters
# ----------------
# pool : list
#   The pool of wells. It is the wells lying within the target cluster
#   as objects.
#
# thickness : ndarray
#   A m x i matrix. m is the number of wells in the database.
#   i the number of fm layers that exists in the database.
#   It carries the information of thickness of each fm layer for
#   every well.
#
# depth : ndarray
#   A m x i matrix. m is the number of wells in the database.
#   i the number of fm layers that exists in the database.
#   It carries the information of TVD top of fm of each layer
#   for every well.
#
# SD_thickness : list
#   A list that holds the standard deviation of thickness for each layer
#   that exists in the input well.
#
# SD_depth : list
#   A list that holds the standard deviation of TVD top of fm for each
#   layer that exists in the input well.
#
# mean_thickness : list
#   A list that holds the mean of thickness for each layer
#   that exists in the input well.
#
# mean_depth : list
#   A list that holds the mean of TVD top of fm for each layer
#   that exists in the input well.

from sklearn.metrics.pairwise import pairwise_distances
import numpy as np

def scaling(pool,thickness, depth, SD_thickness, SD_depth,
            mean_thickness, mean_depth):
    input_matrix = []
    for i in range(len(pool)):
        row_temp = []
        row_temp2 = []
        count = 0
        for j in pool[0].fm:
            scale_thickness = (thickness[pool[i].index,j] -
                               mean_thickness[count]) / SD_thickness[count]
            scale_depth = (depth[pool[i].index,j] - mean_depth[count]) \
                          / SD_depth[count]
            row_temp.append(scale_thickness)
            row_temp2.append(scale_depth)
            count += 1
        row_temp.extend(row_temp2)
        input_matrix.append(row_temp)
    input_matrix = np.asarray(input_matrix)
    return input_matrix