######## Function SD ########
## Calculates the standard deviation in thickness and TVD of
# top of fm for each layer. We only evaluate fm layers that the
# target well has in its geological profile.
# Example: When looking at a layer, say, Nordland. It will take
# the information for thickness in that layer, for all wells.
# Calculate the mean thickness of Nordland. Then, calculate the
# standard deviation in Nordland. This is done for all layers
# that exists in the input well. The same is done with TVD of
# top fm. The standard deviations are later used to standardize
# the data (scale).

# Parameters
# ----------------
# thickness : ndarray
#   A m x i matrix. m is the number of wells in the database.
#   i the number of fm layers that exists in the database.
#   It carries the information of thickness of each fm layer for
#   every well.
#
# depth : ndarray
#   A m x i matrix. m is the number of wells in the database.
#   i the number of fm layers that exists in the database.
#   It carries the information of TVD top of fm of each layer
#   for every well.
#
# pool : list
#   The pool of wells. It is the wells lying within the target cluster
#   as objects.


import numpy as np

def SD(thickness,depth,pool):
    n = len(pool)
    count = 0
    mean_thickness = []
    mean_depth = []
    SD_thickness = []
    SD_depth = []
# Calculate mean values
    for i in pool[0].fm:
        temp_thickness = []
        temp_depth = []
        for j in range(len(pool)):
            temp_thickness.append(thickness[pool[j].index,i])
            temp_depth.append(depth[pool[j].index,i])
        mean_thickness.append(sum(temp_thickness)/ n)
        mean_depth.append(sum(temp_depth) / n)
# Calculate standard deviations
    for i in pool[0].fm:
        denominator_thickness = []
        denominator_depth = []
        for j in range(len(pool)):
                denominator_thickness.append((thickness[pool[j].index,i]-
                                              mean_thickness[count])**2)
                denominator_depth.append((depth[pool[j].index,i]-
                                          mean_depth[count])**2)
        count +=1
        SD_thickness.append(np.sqrt(sum(denominator_thickness) / (n-1)))
        SD_depth.append(np.sqrt(sum(denominator_depth) / (n-1)))
    return SD_thickness, SD_depth, mean_thickness, mean_depth